<?php

namespace Ranig;

use ManiaControl\Callbacks\CallbackListener;
use ManiaControl\Callbacks\CallbackManager;
use ManiaControl\Callbacks\Callbacks;
use ManiaControl\Callbacks\Structures\TrackMania\OnWayPointEventStructure;
use ManiaControl\Callbacks\Structures\TrackMania\OnScoresStructure;
use ManiaControl\Commands\CommandListener;
use ManiaControl\Logger;
use ManiaControl\ManiaControl;
use ManiaControl\Plugins\Plugin;
use ManiaControl\Statistics\StatisticCollector;
use ManiaControl\Statistics\StatisticManager;
use ManiaControl\Utils\Formatter;
use Maniaplanet\DedicatedServer\Connection;
use Maniaplanet\DedicatedServer\Structures\Mod;

/**
 */
class ForceModPlugin implements Plugin, CallbackListener, CommandListener {
	/*
	 * Constants
	 */
	const PLUGIN_ID           = 140;
	const PLUGIN_VERSION      = 2;
	const PLUGIN_NAME         = 'Force Mod Plugin';
	const PLUGIN_AUTHOR       = 'Ranig';
	const SETTING_OVERRIDE    = 'Override existing mod';
	const SETTING_ENVIRONMENT = 'Current server environment';
	const SETTING_MOD_URL     = 'URL of the mod to use';

	/*
	 * Private properties
	 */
	/** @var ManiaControl $maniaControl * */
	private $maniaControl = null;

	/**
	 * @see \ManiaControl\Plugins\Plugin::prepare()
	 */
	public static function prepare(ManiaControl $maniaControl) {
	}

	/**
	 * @see \ManiaControl\Plugins\Plugin::getId()
	 */
	public static function getId() {
		return self::PLUGIN_ID;
	}

	/**
	 * @see \ManiaControl\Plugins\Plugin::getName()
	 */
	public static function getName() {
		return self::PLUGIN_NAME;
	}

	/**
	 * @see \ManiaControl\Plugins\Plugin::getVersion()
	 */
	public static function getVersion() {
		return self::PLUGIN_VERSION;
	}

	/**
	 * @see \ManiaControl\Plugins\Plugin::getAuthor()
	 */
	public static function getAuthor() {
		return self::PLUGIN_AUTHOR;
	}

	/**
	 * @see \ManiaControl\Plugins\Plugin::getDescription()
	 */
	public static function getDescription() {
		return "Forces the mod on the server maps (one environment only)";
	}

	/**
	 * @see \ManiaControl\Plugins\Plugin::load()
	 */
	public function load(ManiaControl $maniaControl) {
		$this->maniaControl = $maniaControl;

		$this->maniaControl->getSettingManager()->initSetting($this, self::SETTING_ENVIRONMENT, 'Stadium');
		$this->maniaControl->getSettingManager()->initSetting($this, self::SETTING_MOD_URL, 'test');
		$this->maniaControl->getSettingManager()->initSetting($this, self::SETTING_OVERRIDE, true);

		// Callbacks
		$this->maniaControl->getCallbackManager()->registerCallbackListener(Callbacks::TM_SCORES, $this, 'handleScores');
	}

	/**
	 * @see \ManiaControl\Plugins\Plugin::unload()
	 */
	public function unload() {
		try {
			if (!$this->maniaControl->getClient()->setForcedMods(false, array())) {
				$this->maniaControl->getChat()->sendInformation('Could not force default mod (unknown error)');
			}
		} catch (Exception $exception) {
			$this->maniaControl->getChat()->sendInformation('Could not force default mod (unknown exception)');
		}
	}
	
	/**
	 * Sets the mod for the next map
	 */
	public function handleScores(OnScoresStructure $fullScoreStructure) {
		if ($fullScoreStructure->getSection() != 'EndMap') {
			return;
		}
		
		$mod = new Mod();
		$mod->env = $this->maniaControl->getSettingManager()->getSettingValue($this, self::SETTING_ENVIRONMENT);
		$mod->url = str_replace(' ', '%20', $this->maniaControl->getSettingManager()->getSettingValue($this, self::SETTING_MOD_URL));

		try {
			$override = $this->maniaControl->getSettingManager()->getSettingValue($this, self::SETTING_OVERRIDE);
			if (!$this->maniaControl->getClient()->setForcedMods($override, $mod)) {
				$this->maniaControl->getChat()->sendInformation('Could not force mod ' . $url . ' (unknown error)');
			}
		} catch (Exception $exception) {
			$this->maniaControl->getChat()->sendInformation('Could not force mod ' . $url . ' (unknown exception)');
		}
	}

}